@@ -48,6 +48,8 @@ class TestSourceCodeGeneration(unittest.TestCase):
         foo = bar = moo
         foo, bar = bar
         foo, bar = foo, bar
+        foo, (bar, moo) = foo
+        ((foo, bar), foo) = moo
 
     @verify
     def testArithmeticAssignment(self):
@@ -250,7 +252,7 @@ class TestSourceCodeGeneration(unittest.TestCase):
         {'a': a, 'b': b, 'c': c}
 
     @verify
-    def testOperators(self):
+    def testComparisons(self):
         foo < bar
         foo > bar
         foo == bar
@@ -259,9 +261,26 @@ class TestSourceCodeGeneration(unittest.TestCase):
         foo <= bar
 
     @verify
-    def testNot(self):
+    def testLogicalOperators(self):
+        foo | bar & bar ^ foo
+        ~ bar
+        ~ (foo or bar)
+    @verify
+    def testOperators(self):
         not foo
-
+        bar or foo
+        not (foo or bar)
+        
     @verify
     def testFormatString(self):
         foo % (bar, moo)
+
+    @verify
+    def testLoop(self):
+        for foo in bar:
+            pass
+        else:
+            pass
+        for foo, (bar, moo) in boo:
+            pass
+        
